from scipy.special import erf

from fipy import *

## NOTE: Distances are in micrometers
## Diffusion coefficient DP is in micrometer^2/s
## Times are in seconds

DP = 1.0e-6 # Diffusion coefficient
cP = 1.0e9  # Surface concentration of phosphorus

OutputDist = 1.0    # Distance at which we will output concentration
ThickM  = 7.0       # thickness of wafer
gridThickM = 1.0e-2 # grid size

nx = int(ThickM/gridThickM) # number of grid points perpendicular to surface

FinalTime = 36000 # total amount of time simulated
dt = 10.0	  # Time step

mesh = Grid1D(dx=gridThickM, nx=nx)

C = CellVariable(mesh=mesh, name="$C$", value=0.0, hasOld=True)
Canalytical = CellVariable(mesh=mesh, name="$C_{analytical}$")

x, = mesh.getCellCenters()
Canalytical.setValue(cP * (1 - erf(x / (2 * sqrt(DP * FinalTime)))))

viewer = Viewer(vars=(C, Canalytical), datamin=0.0, datamax=cP)
viewer.plot()

BCs = (FixedValue(mesh.getFacesLeft(), value=cP))

## CORE OF THE CODE IS THE NEXT EQUATION
eq1 = TransientTerm() == DiffusionTerm(DP)
t = 0
print "dt=", dt
print "D_P=", DP
while t <= FinalTime:
    t=t+dt
    C.updateOld()
    eq1.solve(C, dt=dt, boundaryConditions=BCs)
    if t % 1800 == 0:
        cm = C([[OutputDist]],order=1)
	## for 2.10, uncomment the following lines and make the viewer.plot() line a comment
        viewer.plot()
	#viewer.plot(filename="%g-plot.png" % t)
	#TSVViewer(vars=C).plot(filename="%g-data.txt" %t)
        print "time=", t, "concentration at output distance=", cm
